// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import os.log
@testable import Logger

final class OSLoggerTests: XCTestCase {

    func testThatAllFunctionsCanBeCalled() {
        let osLogger = OSLogger()
        osLogger.debug("Hello!")
        osLogger.default("Hello!")
        osLogger.info("Hello!")
        osLogger.error("Hello!")
        osLogger.fault("Hello!")
    }

    func testMinimumLogLevels() {
        let osLogger = OSLogger()
        osLogger.config.minimumLogLevel = .default
        osLogger.assertShouldPrint(trueLevels: [.default, .debug, .info, .error, .fault], falseLevels: [])

        osLogger.config.minimumLogLevel = .debug
        osLogger.assertShouldPrint(trueLevels: [.debug, .info, .error, .fault], falseLevels: [.default])

        osLogger.config.minimumLogLevel = .info
        osLogger.assertShouldPrint(trueLevels: [.info, .error, .fault], falseLevels: [.default, .debug])

        osLogger.config.minimumLogLevel = .error
        osLogger.assertShouldPrint(trueLevels: [.error, .fault], falseLevels: [.default, .debug, .info])

        osLogger.config.minimumLogLevel = .fault
        osLogger.assertShouldPrint(trueLevels: [.fault], falseLevels: [.default, .debug, .info, .error])
    }

    func testLogMessage() {
        let triLogger = OSLogger()
        triLogger.config.isDevelopmentInfoEnabled = true
        let message = "Test test"
        let level = OSLogType.debug
        let fileName = "/Users/test/ATestFile.swift"
        let functionName = "aTestFunction"
        let line: UInt = 1337
        let log = triLogger.constructLogMessage(message, at: level, file: fileName, function: functionName, line: line)
        XCTAssertEqual(log, "🐛 ATestFile.swift:1337 - aTestFunction | Test test")

        triLogger.config.isDevelopmentInfoEnabled = false
        let log2 = triLogger.constructLogMessage(message, at: level, file: fileName, function: functionName, line: line)
        XCTAssertEqual(log2, "🐛 Test test")
    }

    func testEmojis() {
        let osLogger = OSLogger()
        osLogger.config.isEmojisEnabled = false
        osLogger.config.isDevelopmentInfoEnabled = false

        let log = osLogger.constructLogMessage("Test test", at: .info, file: "fileName", function: "functionName", line: 1337)
        XCTAssertEqual(log, "[INFO] Test test")

        osLogger.config.isEmojisEnabled = true
        let log2 = osLogger.constructLogMessage("Test test", at: .info, file: "fileName", function: "functionName", line: 1337)
        XCTAssertEqual(log2, "ℹ️ Test test")
    }

    func testLogLevelData() {
        XCTAssertEqual(OSLogType.default.emoji, "📢")
        XCTAssertEqual(OSLogType.default.title, "DEFAULT")

        XCTAssertEqual(OSLogType.debug.emoji, "🐛")
        XCTAssertEqual(OSLogType.debug.title, "DEBUG")

        XCTAssertEqual(OSLogType.info.emoji, "ℹ️")
        XCTAssertEqual(OSLogType.info.title, "INFO")

        XCTAssertEqual(OSLogType.error.emoji, "🔥")
        XCTAssertEqual(OSLogType.error.title, "ERROR")

        XCTAssertEqual(OSLogType.fault.emoji, "💥")
        XCTAssertEqual(OSLogType.fault.title, "FAULT")
    }

    static var allTests = [
        ("testThatAllFunctionsCanBeCalled", testThatAllFunctionsCanBeCalled),
        ("testMinimumLogLevels", testMinimumLogLevels),
        ("testLogMessage", testLogMessage),
        ("testEmojis", testEmojis),
        ("testLogLevelData", testLogLevelData),
    ]
}


private extension OSLogger {
    func assertShouldPrint(trueLevels: [OSLogType], falseLevels: [OSLogType]) {
        for trueType in trueLevels {
            XCTAssertTrue(shouldPrint(trueType), "True type (\(trueType.title)) failed for \(config.minimumLogLevel.title)")
        }
        for falseType in falseLevels {
            XCTAssertFalse(shouldPrint(falseType), "False type (\(falseType.title)) failed for \(config.minimumLogLevel.title)")
        }
    }
}
